// Convert.cpp - NESTrade
// Conversion routines to a similar file format.

#include "convert.h"
#include <stdio.h>				// for FILE
#include <memory.h>				// for memcpy()
#include "../osd_cpu.h"			// for types

//***********************************************************
// Genesis SMD (interleaved w/header) to Bin (raw) conversion
// Note: .MD and .MGD formats are not included.
//********************************
// from Genesis_ROM_Format.txt
// Credits (as they appear in the document:
//Felipe XnaK: 
//  http://www.classicgaming.com/launchtool
//  felipe@ComPorts.com
//Volker Oth (dOnut):
//  http://members.aol.com/~volkeroth
//  volkeroth@aol.com
//
//Also used as a reference: DiskDude's "genesisr.txt" from "gentech.zip"
//provides Pascal source for bin to SMD and SMD to bin converter.
// Credits as they appear in the document:
//Turtle Group Incorporated - http://tgi.home.ml.org/
//
//Tested against Kaja's "smd.com" v 0.3

UINT32 SMD2BIN( unsigned char *SMDImage, UINT32 iImageLength ) {		// in-place SMD to BIN
	const int iBlockSize = 16*1024;
	unsigned char *SMDBlock = SMDImage + 512;
	unsigned char *SMDImage1 = SMDBlock;
	unsigned char *SMDImage2 = (unsigned char *) &SMDImage[iBlockSize/2] + 512;		// skip initial header

	if( SMDImage[1] != 0x3 || SMDImage[8] != 0xAA || SMDImage[9] != 0xBB ) {
		return 0;
	} else {
		unsigned char *BINblock = new unsigned char[iBlockSize];
		for( int iBlockNum = 0; iBlockNum < SMDImage[0]; iBlockNum++) {
			for( int iOffset = 0; iOffset < iBlockSize; ) {			// iOffset is increased in assignments
				BINblock[iOffset++] = *SMDImage2++;
				BINblock[iOffset++] = *SMDImage1++;
			}
			memcpy( SMDBlock, BINblock, iBlockSize );
			SMDBlock += iBlockSize;				// SMDBlock = iBlockSize * iBlockNum;
			SMDImage1 += iBlockSize/2;			//SMDImage1 = SMDBlock;
			SMDImage2 += iBlockSize/2;			//SMDImage2 = SMDBlock + iBlockSize / 2;
		}
	}

	return 1;
}

//***********************************************************
