#include <stdio.h>
#include <string.h>
#include <vector>
#include "../id.h"			// for external szExt declaration

std::vector<tagInfo> vInfo;
char *szExt = 0;		// definition here should satisfy compiler
char *szSortDir = 0;	// definition here should satisfy compiler

int unLF( char *p ) {
	// remove lf from string (fgets does not remove)
	// TODO: does this work for all OS?
	// break on the first non-HT, LF, FF, CR
	int i;

	i = strlen( p );
	while( --i ) {
		if( p[i] > 8 && p[i] < 14 )
			p[i] = '\0';
		else
			break;
	}
	return 1;
	// TODO: return strlen?  what would be useful? return char *?
}

char *AddExts ( char *p ) {	// space-separated null-term
	size_t x = 0;

	unLF( p );
#ifndef JSORT
	// hack to have extensions at the top of the dat file - redirectable
	fprintf(stdout, "%s \n", p);
#endif

	while(p[x]) {
		if(p[x] == ' ')
			p[x] = '\0';
		x++;
		if(p[x] > 8 && p[x] < 14)	// HT, LF, VT, FF, CR
			break;
	}

	szExt = new char[x+2];	// need double null term
	if( szExt ) {
		memcpy(szExt, p, x);
		szExt[x++] = '\0';
		szExt[x] = '\0';
	}
	return szExt;	
}

unsigned MatchExt(char *cTmpext, char *ext) {
	int j=0;
	int c,d;
	while(( c=tolower( ext[j] ) ) && ( d=tolower( cTmpext[j] ) )) {
		if(c != d) {
			j = 0;
			break;
		}
		j++;
	}
	return j;	// True if matched else false
}

int MatchExtToArray(char *sztmpExt) {
	int x = 0;
	int iRet = 0;

	if(!strcmp( &szExt[0], "*.*" ))
		return 1;						// no header or filetype
	while(szExt[x]) {
		if(szExt[x] == '*')
			x++;
		if(MatchExt( &szExt[x], sztmpExt )) {
			iRet = 1;
			break;
		}
		while(szExt[x++]);
	}
	
	return iRet;
}

void UnloadDat( void ) {
	vInfo.clear();			// call erase() for each element
}

int LoadDat( char *pszSystem ) {
	FILE *fp;
	tagInfo ri;						// temp struct
	char szTemp[MAX_BUFF];			// to hold CRC and name data

#ifdef JSORT
	char szTokSep[] = " \n\r";
#else
	char szTokSep[] = "\xAC\n\r";	// for RomCenter
#endif

	char *p;
	int iRet = 0;					// 0 == fail, number loaded

	szSortDir = new char[strlen( pszSystem ) + 1];
	strcpy( &szSortDir[0], pszSystem );

	strcpy( &szTemp[0], pszSystem );
#ifdef JSORT
	strcat( &szTemp[0], ".crc" );
#else
	strcat( &szTemp[0], ".dat" );
#endif
	if( fp = fopen(&szTemp[0], "r")) {

		vInfo.reserve(MAX_RESERVE);

		if(fgets( &szTemp[0], MAX_BUFF, fp)) {

#ifdef JSORT
			if(!szExt)				// else use command-line
				AddExts("*.jpg");			// this must exist
#else
			if( szTemp[0] == '[' ) {
				if(!szExt)				// else use command-line
					AddExts( "*.*" );
				do {
					unLF( &szTemp[0] );
					if(!strcmp( &szTemp[0], "[GAMES]"))
						break;
					fgets( &szTemp[0], MAX_BUFF, fp);
				} while	(1);			// advances to GAMES section
			} else {
				if(!szExt)				// else use command-line
					AddExts( &szTemp[0] );
			}
#endif

			while(fgets( &szTemp[0], MAX_BUFF, fp)) {
				if( szTemp[0] == '\xAC' ) {		// RomCenter datfile
					ri.bHave = 0;
					// ignore first field - lower case usually?
					p = strtok( &szTemp[1], &szTokSep[0] );
					// use this as the filename - no extensions
					p = strtok( NULL, &szTokSep[0] );
					strncpy( &ri.szName[0], p, FNLEN );
					// clone name, ignore
					p = strtok( NULL, &szTokSep[0] );
					// clone full name, ignore
					p = strtok( NULL, &szTokSep[0] );
					// filename, ignore.  should be field 1+extension
					p = strtok( NULL, &szTokSep[0] );
					// CRC at last
					p = strtok( NULL, &szTokSep[0] );
					ri.crc = strtoul( p, NULL, 0x10 );
					vInfo.push_back( ri );
					iRet++;
				} else if(p = strchr( &szTemp[0], ' ' )) {
					ri.bHave = 0;
					*p = '\0';
					p++;
					ri.crc = strtoul( &szTemp[0], NULL, 0x10 );	// 0x10 = hex (base 16)
					unLF( p );
							
					strncpy( &ri.szName[0], p, FNLEN );
					vInfo.push_back( ri );
					iRet++;
				}			// else failure

#ifdef _DEBUG
				if(!ri.crc)
					fprintf(stderr, "CRC is null: %s\n", &szTemp[0]);
#endif	// _DEBUG
			}
		} else				// initial fgets() failure
			szExt = 0;

		fclose(fp);
	}
	return iRet;
}

tagInfo *IdProc( unsigned int iCRC) {
	tagInfo *p = 0;
	std::vector<tagInfo>::iterator itInfo = vInfo.begin();
	while( itInfo < vInfo.end() ) {
		if(itInfo->crc == iCRC) {
			p = itInfo;
			break;
		}
		itInfo++;
	}
	return p;
}
