// This source can make either RomSort or JSort.  Small
// differences are done with compiler flags (below) and
// bigger differences are done with the source tree.  I
// tried to make id.cpp as similar as possible but this
// is becoming too different - giving up... They should
// be the same except for the loop which starts:
//			while(fgets( &szTemp[0], MAX_BUFF, fp)) {
// so it's easier to maintain

// RomSort - need to link with zlib.

// Compiler Flags:
// ROMCENTER    - include only the header-skipping routines
//                compatible with RomCenter.  Actually only
//                enables/disables file extension compares.
// NOSKIPGENHDR - include 512-byte Genesis header.  GoodGen
//                and RomCenter include it, so RomSort does
//                but you can disable it.
// JSORT        - make JSort util instead of RomSort.  This
//                is a lot smaller.  It does not need to be
//                linked with zlib
// JSHERIFF     - for JSort, use whole-file CRCs instead of
//                image-only to be compatible with the crcs
//                of Jpeg Sheriff.  Probably a bit faster -
//                but it ignores the format segments of the
//                the JFIF/JPEG format.  If not defined, it
//                will not work with JSheriff dats.

// note: szArg in GetOpts() is used when we can actually
// allocate and write to global szSortDir, and not have
// to add it in LoadDat().  But I like it in LoadDat()
// because it's only set if valid (if the file opens)

// note: the only times you have to worry about path
// separator '/' or '\' should be in the /arch/$(OS)
// tree. main() truncates the dir off of argv[0] (so
// if it is not run with the full path you will have
// to add a hack like the '#ifdef USE_WIN' and if so
// you can deal with the separator.  main will check
// for either back or foreward as it is though.


#define STRICT

#ifdef _DEBUG
	#define VERSION "v1.0 (Debug)"
#else
	#define VERSION "v1.0 (Release)"
#endif

#ifdef JSORT
	#define APPNAME "JSort"
#else
	#define APPNAME "RomSort"
#endif

#ifdef _WIN32				// hack: check for windows compilers
	#define WIN32_LEAN_AND_MEAN
	#include <windows.h>
	#define USE_WIN			// for windows GetCurrentDirectory since argv[0] does not always have full path
#else	// include system dependant files here if needed for some hack

#endif

#include "dir.h"
#include "dir_ops.h"
#include "crc32.h"			// generate CRC table on load
#include "sort.h"			// main processing

#include "id.h"				// for LoadDat(), UnloadDat()
#include "opts.h"			// options are bit-masked
#include <stdio.h>
#include <string.h>

#ifndef _MAX_PATH
#define _MAX_PATH 255
#endif


void Usage(void) {
// TODO: change for MAC or non-commandline systems
	fprintf( stderr,"---------------------------------------------\n");
	fprintf( stderr,"Usage   : " APPNAME " [-d] [system] [extensions]\n\n");
	fprintf( stderr,"Options : uses [system].dat to rename\n");
	fprintf( stderr,"          -d   Dump CRC info (make a dat)\n");
#ifdef JSORT
	fprintf( stderr,"\nEx: To rename from 'bc_all14.crc', type '" APPNAME " bcall_14'\n");
	fprintf( stderr,"    To make 'bc_all14.crc', type '" APPNAME " -d *.jpg > bcall_14.crc'\n");
#else
	fprintf( stderr,"\nEx: To rename Atari 2600, type '" APPNAME " 2600'\n");
	fprintf( stderr,"    To make a 2600 dat, type '" APPNAME " -d *.a26 *.bin > 2600.dat'\n");
#endif
}

int GetOpts(int argc, char *argv[], char *szArg) {

	int opts = 0;
	int x;
	char szTemp[255];

	for(x = 1; x < argc; x++) {
		switch(argv[x][0]) {
		case '-':
			switch(argv[x][1]) {
			case 'd':
			case 'D':
				opts |= OPT_DUMP;
				if( argv[++x][0] ) {
//					strcpy(szArg, argv[x++]);	// if first arg is datfile name
					szTemp[0] = '\0';
					while( argv[x]) {		// make space-separated string to pass
						strcat( &szTemp[0], argv[x] );
						strcat( &szTemp[0], " " );
						x++;
					}
					AddExts( &szTemp[0] );			// id.cpp
					argc = 0;					// no more options
				} else {
					argc = 0;					// err, no outfile
					opts = 0;
				}
				break;
			default:						// any invalid switch
				argc = 0;					// break
				opts = 0;
			}
			break;
		default:
			strcpy(szArg, argv[x]);
			opts |= OPT_FILE;
		}
	}

	if(!opts)
		Usage();
	else {
		opts |= OPT_RECURSE;				// default
		if( !(opts & OPT_DUMP))
			opts |= OPT_RENAME;
	}

	return(opts);							// opts must be 0 for error
}


int main(int argc, char *argv[]) {

	int i, opts;
	char szDir[_MAX_PATH];
	char szArg[_MAX_PATH];

	// this line just makes it easier to pass &sort_proc()
	int (*pfn) (char *szFile, char *szDir, int iOpts) = sort_proc;

	fprintf( stderr, APPNAME " " VERSION "\n");

#ifdef USE_WIN
	GetCurrentDirectory(_MAX_PATH, szDir);
	strcat((char *) &szDir, "\\");
#else								// three lines replace GetCurrentDirectory()
	char *p = strrchr(argv[0], '/');
	if(!p)
		p = strrchr(argv[0], '\\');	// just in case
	if(p) {
		p[1] = 0;
		strcpy(szDir, argv[0]);		// should preserve back or forward slash
	}
#endif

	opts = GetOpts( argc, argv, &szArg[0] );

	if(opts) {
		crcgen();
		if( opts & OPT_RENAME && opts & OPT_FILE ) {	// dump does not require dat
			i=LoadDat( &szArg[0]);						//calls LoadHave
		} else
			i = -1;
		if(i) {
			if(i == -1) {				// important to have version
				fprintf(stderr, "Dump mode\n\n");
			} else
#ifdef JSORT
				fprintf(stderr, "%u files from %s.crc.\n\n", i, &szArg[0]);
#else
				fprintf(stderr, "%u files from %s.dat.\n\n", i, &szArg[0]);
#endif

			if(opts & OPT_RECURSE) {
				// szExt is double-null terminated
				int x = 0;
				while(szExt[x]) {
					walkdir(&szDir[0], pfn, &szExt[x], opts );
					while(szExt[x++]);
				}
#ifndef JSORT
				if( !strstr( &szExt[0], "*.*") && !strstr( &szExt[0], "*.zip"))
					walkdir(&szDir[0], pfn, "*.zip", opts );
#endif	// JSORT
			}	// TODO: else { }

			if( i>0 ) {			// dump does not require dat
				fprintf( stderr, "\nUnloading Data\n");
				UnloadDat();	// calls WriteHave()
			}
			if(szExt)
				delete [] szExt;
			if(szSortDir)
				delete [] szSortDir;

		} else {
			fprintf( stderr,"Error getting data.\n");
		}
	}
	return 0;
}
